<?php
/*
Plugin Name: WebP Images
Plugin URI: https://totalpress.org/plugins/webp-images?utm_source=wp-dashboard&utm_medium=installed-plugin&utm_campaign=webp-images
Description: Convert and compress images in WebP format easily. Speed up your website.
Author: TotalPress.org
Author URI: https://totalpress.org/?utm_source=wp-dashboard&utm_medium=installed-plugin&utm_campaign=webp-images
Text Domain: webp-images
Domain Path: /languages/
Version: 1.0.5
*/

if (!defined('ABSPATH')) {
    die('Invalid request.');
}

define('WEBP_IMAGES_BULK_TIME', 5); // Time between bulk actions
define('WEBP_IMAGES_BULK_ELEMENTS', 10); // Elements for single bulk action


require plugin_dir_path(__FILE__) . '/vendor/autoload.php';

use WebPConvert\WebPConvert;

require plugin_dir_path(__FILE__) . '/inc/functions.php';

if (!class_exists('webp_images')) {
    class webp_images
    {

        public function __construct()
        {
            //
        }

        public function init_plugin()
        {
            add_action('init', array($this, 'init_load_textdomain'));
            register_activation_hook(__FILE__, array($this, 'plugin_enable'));
            register_deactivation_hook(__FILE__, array($this, 'plugin_reset'));
            add_action('admin_menu', array($this, 'init_menu_page'));
            add_action('admin_enqueue_scripts', array($this, 'init_admin_enqueue'));
            add_action('init', array($this, 'dismiss_compatibility_notice'));
            add_action('init', array($this, 'start_bulk_conversion'));
            add_action('init', array($this, 'start_bulk_remove'));
            add_action('admin_notices', array($this, 'init_admin_notices'));
            add_filter('plugin_action_links', array($this, 'init_plugin_action_links'), 10, 2);
            add_action('init', array($this, 'force_stop_process'));
            add_action('wp_ajax_webp_images_countdown', array($this, 'webp_images_countdown'));
            // CRON
            add_filter('cron_schedules', array($this, 'add_cron_schedules'));
            add_action('init_bulk_webp_conversion', array($this, 'bulk_webp_conversion'));
            add_action('init_bulk_webp_remove', array($this, 'bulk_webp_remove'));
        }

        public function init_load_textdomain()
        {
            load_plugin_textdomain('webp-images', false, dirname(plugin_basename(__FILE__)) . '/languages');
        }

        public function is_pro_version_active()
        {
            $return = false;
            $pro_version = in_array('webp-images-pro/webp-images-pro.php', apply_filters('active_plugins', get_option('active_plugins')));
            if ($pro_version) {
                $return = true;
            }
            return $return;
        }

        public function is_apache()
        {
            $return = false;
            $software = isset($_SERVER['SERVER_SOFTWARE']) ? $_SERVER['SERVER_SOFTWARE'] : false;
            if ($software && strpos($software, 'Apache') !== false) {
                $return = true;
            }
            return $return;
        }

        public function htaccess_is_ok()
        {
            if (!function_exists('get_home_path')) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
            }
            $return = false;
            $htaccess = get_home_path() . ".htaccess";
            if (file_exists($htaccess) && is_writable($htaccess)) {
                $return = true;
            }
            return $return;
        }

        public function is_compatible()
        {
            $return = false;
            if (webp_images_getCwebpStatus()[0] == true) {
                $return = true;
            }
            if (webp_images_getImageMagickStatus()[0] == true) {
                $return = true;
            }
            if (webp_images_getVipsStatus()[0] == true) {
                $return = true;
            }
            if (webp_images_getImagickStatus()[0] == true) {
                $return = true;
            }
            if (webp_images_getGraphicsMagickStatus()[0] == true) {
                $return = true;
            }
            if (webp_images_getGdStatus()[0] == true) {
                $return = true;
            }
            return $return;
        }

        public function init_plugin_action_links($links, $file)
        {
            if ($file == 'webp-images/webp-images.php') {
                $links[] = sprintf('<a href="%s" target="_blank"> %s </a>', 'https://wordpress.org/support/plugin/webp-images', __('Support', 'webp-images'));
                $links[] = sprintf('<a href="%s"> %s </a>', admin_url('upload.php?page=webp-images-settings'), __('Settings', 'webp-images'));
                if (!$this->is_pro_version_active()) {
                    $links[] = sprintf('<a href="%s" style="font-weight: bold;"> %s </a>', 'https://www.andreadegiovine.it/risorse/plugin/webp-images?utm_source=tools_plugin_page&utm_medium=plugin_page&utm_campaign=webp_images', __('Get PRO', 'webp-images'));
                }
            }
            return $links;
        }

        public function init_admin_notices()
        {

            if (!$this->is_compatible() && get_option('webp_images_compatibility_notice', 0) == 0) { ?>
                <div class="notice notice-error is-dismissible">
                    <p><?php _e('<strong>WebP Images</strong> it appears that your server does not (or partially) have the required features.<br>If errors or problems occur while converting images, contact your provider to enable at least one of the required features.', 'webp-images'); ?></p>
                    <p><a href="<?php echo admin_url('upload.php?page=webp-images-settings&action=status'); ?>" class="button button-secondary"><?php _e('Check system status', 'webp-images'); ?></a> <a href="<?php echo add_query_arg(array('action' => 'dismiss_webp', 'token' => wp_create_nonce('webp-images-dismiss')), admin_url('upload.php?page=webp-images-settings')); ?>" class="button button-primary"><?php _e('Ok! I know', 'webp-images'); ?></a></p>
                </div>
            <?php }
            if (!$this->is_apache()) { ?>
                <div class="notice notice-error is-dismissible">
                    <p><?php _e('<strong>WebP Images</strong> works only on Apache servers.<br>Usually all Linux servers / hosting use this software, Windows servers / hosting are not supported. Ask your provider.', 'webp-images'); ?></p>
                    <p><a href="https://www.apache.org" class="button button-secondary" target="_blank" rel="nofollow"><?php _e('Read more about Apache', 'webp-images'); ?></a></p>
                </div>
            <?php }
            if (!$this->htaccess_is_ok()) { ?>
                <div class="notice notice-error is-dismissible">
                    <p><?php _e('<strong>WebP Images</strong> requires that .htaccess file exists in the root of the WordPress installation, and that it is writable.<br>Check if the file exists (via FTP or other file manager) and if it has the correct write permissions (644 for files).<br>To automatically generate this file, visit the "Settings > Permalinks" page and save the settings without making any changes.', 'webp-images'); ?></p>
                    <p><a href="<?php echo admin_url('options-permalink.php'); ?>" class="button button-primary"><?php _e('Save permalinks', 'webp-images'); ?></a> <a href="https://wordpress.org/support/article/changing-file-permissions/" class="button button-secondary" target="_blank" rel="nofollow"><?php _e('WordPress File Permissions', 'webp-images'); ?></a></p>
                </div>
            <?php }
            if (wp_next_scheduled('init_bulk_webp_conversion')) { ?>
                <div class="notice notice-success is-dismissible">
                    <p><?php printf(__('<strong>WebP Images</strong> is converting your images to webp.<br><big><u><span class="webp-images-conversion-countdown">%s</span> images left</u></big> to complete the conversion.', 'webp-images'), $this->count_not_webp_media_images()); ?></p>
                    <p><a href="<?php echo admin_url('upload.php?page=webp-images-settings'); ?>" class="button button-secondary"><?php _e('WebP settings', 'webp-images'); ?></a> <a href="<?php echo add_query_arg(array('action' => 'stop_webp', 'token' => wp_create_nonce('webp-images-stop')), admin_url('upload.php?page=webp-images-settings')); ?>" class="button button-primary"><?php _e('Stop process', 'webp-images'); ?></a></p>
                </div>
            <?php } elseif (wp_next_scheduled('init_bulk_webp_remove')) { ?>
                <div class="notice notice-warning is-dismissible">
                    <p><?php printf(__('<strong>WebP Images</strong> is removing all webp versions of your images.<br><big><u><span class="webp-images-remove-countdown">%s</span> images left</u></big> to complete the removal.', 'webp-images'), $this->count_webp_media_images()); ?></p>
                    <p><a href="<?php echo admin_url('upload.php?page=webp-images-settings'); ?>" class="button button-secondary"><?php _e('WebP settings', 'webp-images'); ?></a> <a href="<?php echo add_query_arg(array('action' => 'stop_webp', 'token' => wp_create_nonce('webp-images-stop')), admin_url('upload.php?page=webp-images-settings')); ?>" class="button button-primary"><?php _e('Stop process', 'webp-images'); ?></a></p>
                </div>
            <?php } else {
                return;
            }
        }

        public function init_menu_page()
        {
            add_submenu_page('upload.php', __('WebP Images settings', 'webp-images'), __('WebP settings', 'webp-images'), 'manage_options', 'webp-images-settings', array($this, 'render_menu_page'));

            add_action('admin_init', array($this, 'register_webp_images_settings'));
        }

        public function register_webp_images_settings()
        {
            register_setting('webp-images-settings-group', 'webp-images-pro_license_key');
            register_setting('webp-images-settings-group', 'webp-images-pro_last_license_check');
            register_setting('webp-images-settings-group', 'webp_images_quality');
            register_setting('webp-images-settings-group', 'webp_images_auto_convert');
        }

        public function render_menu_page()
        {
            $main_page = menu_page_url('webp-images-settings', false);
            $status_page = $main_page . '&action=status';
            $request_page = isset($_GET['action']) && !empty($_GET['action']) && in_array($_GET['action'], array('status')) ? $_GET['action'] : 'main';
            ?>
            <div class="wrap webp-images-plugin">
                <h1><?php _e('WebP Images settings', 'webp-images'); ?></h1>

                <div class="webp-images-container">
                    <div class="webp-images-main">

                        <nav class="nav-tab-wrapper wp-clearfix" aria-label="Secondary menu">
                            <a href="<?php echo $main_page; ?>" class="nav-tab<?php echo ($request_page == 'main' ? ' nav-tab-active' : ''); ?>"><?php _e('General Settings', 'webp-images'); ?></a>
                            <a href="<?php echo $status_page; ?>" class="nav-tab<?php echo ($request_page == 'status' ? ' nav-tab-active' : ''); ?>"><?php _e('Check status', 'webp-images'); ?></a>
                        </nav>
                        <?php include(plugin_dir_path(__FILE__) . 'parts/options-page-' . $request_page . '.php'); ?>
                    </div>
                    <div class="webp-images-sidebar">
                        <?php include(plugin_dir_path(__FILE__) . 'parts/sidebar.php'); ?>
                    </div>
                </div>
            </div>
<?php
        }

        public function init_admin_enqueue()
        {
            wp_register_style('admin-webp-images', plugin_dir_url(__FILE__) . 'assets/admin-ui.css', false, '1.0.0');
            wp_enqueue_style('admin-webp-images');

            wp_register_script('admin-webp-images-js', plugin_dir_url(__FILE__) . 'assets/admin.js', array('jquery'));
            wp_localize_script('admin-webp-images-js', 'WebpImages', array(
                'ajaxurl' => admin_url('admin-ajax.php'),
                'nonce' => wp_create_nonce('webp-images-countdown'),
            ));
            wp_enqueue_script('jquery');
            wp_enqueue_script('admin-webp-images-js');
        }

        public function estimated_conversion_time()
        {
            $conversion_time = 0;

            $count_images_not_converted = $this->count_not_webp_media_images();
            $max_elements_for_bulk = (int) apply_filters('webp_images_bulk_max_elements', WEBP_IMAGES_BULK_ELEMENTS);
            $time_between_bulk_actions = (int) apply_filters('webp_images_bulk_time_between', WEBP_IMAGES_BULK_TIME);

            $conversion_time = ($count_images_not_converted / $max_elements_for_bulk) * $time_between_bulk_actions;

            $time = (ceil($conversion_time) % $time_between_bulk_actions === 0) ? ceil($conversion_time) : round(($conversion_time + $time_between_bulk_actions / 2) / $time_between_bulk_actions) * $time_between_bulk_actions;
            return $time;
        }

        public function get_media_images()
        {
            $args = array(
                'post_type' => 'attachment',
                'post_mime_type' => array(
                    'jpg|jpeg|jpe' => 'image/jpeg',
                    //'gif' => 'image/gif',
                    'png' => 'image/png',
                ),
                'post_status' => 'inherit',
                'posts_per_page' => -1,
            );
            $all_images = get_posts($args);
            return $all_images;
        }

        public function get_not_webp_media_images($limit = -1)
        {
            $return_array = array();

            $args = array(
                'post_type' => 'attachment',
                'orderby' => 'rand',
                'order' => 'ASC',
                'post_mime_type' => array(
                    'jpg|jpeg|jpe' => 'image/jpeg',
                    //'gif' => 'image/gif',
                    'png' => 'image/png',
                ),
                'meta_query' => array(
                    array(
                        'key' => 'webp_src',
                        'compare' => 'NOT EXISTS'
                    )
                ),
                'post_status' => 'inherit',
                'posts_per_page' => $limit,
            );
            $all_images = get_posts($args);

            foreach ($all_images as $image) {
                $image_id = $image->ID;
                $image_path = get_attached_file($image_id);
                $image_file = basename($image_path);
                $return_array[$image_id][$image_file] = $image_path;
                $image_sizes = get_post_meta($image_id, '_wp_attachment_metadata', true);
                $sizes = isset($image_sizes['sizes']) ? $image_sizes['sizes'] : false;
                if ($sizes) {
                    foreach ($sizes as $size => $value) {
                        $resized_image_path = dirname($image_path) . '/' . $value['file'];
                        if (!file_exists($resized_image_path)) {
                            continue;
                        }
                        $return_array[$image_id][$value['file']] = $resized_image_path;
                    }
                }
            }
            return $return_array;
        }

        public function get_webp_media_images($limit = -1)
        {
            $return_array = array();

            $args = array(
                'post_type' => 'attachment',
                'post_mime_type' => array(
                    'jpg|jpeg|jpe' => 'image/jpeg',
                    //'gif' => 'image/gif',
                    'png' => 'image/png',
                ),
                'meta_query' => array(
                    array(
                        'key' => 'webp_src',
                        'value' => 'done',
                    ),
                ),
                'post_status' => 'inherit',
                'posts_per_page' => $limit,
            );
            $all_images = get_posts($args);
            foreach ($all_images as $image) {
                $image_id = $image->ID;
                $image_path = get_attached_file($image_id);
                $image_file = basename($image_path);
                $return_array[$image_id][$image_file] = $image_path;
                $image_sizes = get_post_meta($image_id, '_wp_attachment_metadata', true);
                $sizes = isset($image_sizes['sizes']) ? $image_sizes['sizes'] : false;
                if ($sizes) {
                    foreach ($sizes as $size => $value) {
                        $resized_image_path = dirname($image_path) . '/' . $value['file'];
                        if (!file_exists($resized_image_path)) {
                            continue;
                        }
                        $return_array[$image_id][$value['file']] = $resized_image_path;
                    }
                }
            }
            return $return_array;
        }

        public function count_media_images()
        {
            $all_images = $this->get_media_images();
            return count($all_images);
        }

        public function count_not_webp_media_images()
        {
            $all_images = $this->get_not_webp_media_images();
            return count($all_images);
        }

        public function count_webp_media_images()
        {
            $all_images = $this->get_webp_media_images();
            return count($all_images);
        }

        public function convert_img_to_webp($original_path = false, $new_path = false, $quality = 80)
        {
            if (!$original_path || !$new_path || !file_exists($original_path) || !is_writable(dirname($new_path))) {
                return false;
            }
            if (file_exists($new_path)) {
                return true;
            }
            if ($this->is_pro_version_active()) {
                $quality = (int) get_option('webp_images_quality', 80);
                if ($quality > 100 || $quality < 1) {
                    $quality = 80;
                }
            }
            $options = array(
                'converters' => array('cwebp', 'vips', 'imagick', 'gmagick', 'imagemagick',  'graphicsmagick', 'wpc', 'ewww', 'gd'),
                'png' => array('quality' => $quality),
                'jpeg' => array('quality' => $quality),
            );
            WebPConvert::convert($original_path, $new_path, $options);
            if (file_exists($new_path)) {
                return true;
            } else {
                return false;
            }
        }

        public function remove_webp_image($webp_path = false)
        {
            if (!$webp_path || !is_writable(dirname($webp_path))) {
                return false;
            }
            if (!file_exists($webp_path)) {
                return true;
            }
            unlink($webp_path);
            return true;
        }

        public function dismiss_compatibility_notice()
        {
            if (!$this->is_apache() || !$this->htaccess_is_ok() || !is_admin() || !isset($_GET['action']) || $_GET['action'] !== 'dismiss_webp' || !isset($_GET['token']) || !wp_verify_nonce($_GET['token'], "webp-images-dismiss")) {
                return;
            }
            update_option('webp_images_compatibility_notice', 1);
            wp_redirect(admin_url('upload.php?page=webp-images-settings&action=status'));
            exit;
        }

        public function start_bulk_conversion()
        {
            if (!$this->is_apache() || !$this->htaccess_is_ok() || !is_admin() || !isset($_GET['action']) || $_GET['action'] !== 'convert_webp' || !isset($_GET['token']) || !wp_verify_nonce($_GET['token'], "webp-images-convert")) {
                return;
            }
            $this->stop_cron_schedule();
            $this->start_cron_schedule('init_bulk_webp_conversion');
            wp_redirect(admin_url('upload.php?page=webp-images-settings'));
            exit;
        }

        public function bulk_webp_conversion()
        {
            $result = 'completed';

            $max_elements_for_bulk = apply_filters('webp_images_bulk_max_elements', WEBP_IMAGES_BULK_ELEMENTS);
            $all_images = $this->get_not_webp_media_images($max_elements_for_bulk);

            if (count($all_images) < 1) {
                $this->stop_cron_schedule();
                return;
            }

            $original_time_limit = ini_get('max_execution_time');
            ini_set('max_execution_time', 0);
            foreach ($all_images as $id => $files) {
                foreach ($files as $name => $path) {
                    $base_path = dirname($path);
                    $make_conversion = $this->convert_img_to_webp($path, $base_path . '/' . $name . '.webp');
                    if (!$make_conversion) {
                        update_post_meta($id, 'webp_src', 'error');
                        $result = 'errors';
                    }
                }
                if ($result == 'completed') {
                    update_post_meta($id, 'webp_src', 'done');
                }
            }
            ini_set('max_execution_time', $original_time_limit);
        }

        public function start_bulk_remove()
        {
            if (!is_admin() || !isset($_GET['action']) || $_GET['action'] !== 'remove_webp' || !isset($_GET['token']) || !wp_verify_nonce($_GET['token'], "webp-images-remove")) {
                return;
            }
            $this->stop_cron_schedule();
            $this->start_cron_schedule('init_bulk_webp_remove');
            wp_redirect(admin_url('upload.php?page=webp-images-settings'));
            exit;
        }

        public function bulk_webp_remove()
        {
            $result = 'completed';

            $max_elements_for_bulk = apply_filters('webp_images_bulk_max_elements', WEBP_IMAGES_BULK_ELEMENTS);
            $all_images = $this->get_webp_media_images($max_elements_for_bulk);

            if (count($all_images) < 1) {
                $this->stop_cron_schedule();
                return;
            }

            $original_time_limit = ini_get('max_execution_time');
            ini_set('max_execution_time', 0);
            foreach ($all_images as $id => $files) {
                foreach ($files as $name => $path) {
                    $base_path = dirname($path);
                    $make_remove = $this->remove_webp_image($base_path . '/' . $name . '.webp');
                    if (!$make_remove) {
                        $result = 'errors';
                    }
                }
                if ($result == 'completed') {
                    delete_post_meta($id, 'webp_src');
                }
            }
            ini_set('max_execution_time', $original_time_limit);
        }

        public function force_stop_process()
        {
            if (!is_admin() || !isset($_GET['action']) || $_GET['action'] !== 'stop_webp' || !isset($_GET['token']) || !wp_verify_nonce($_GET['token'], "webp-images-stop")) {
                return;
            }
            $this->stop_cron_schedule();
            wp_redirect(admin_url('upload.php?page=webp-images-settings'));
            exit;
        }

        public function write_htaccess_rules()
        {
            if (!function_exists('get_home_path')) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
            }
            if (!function_exists('insert_with_markers')) {
                require_once ABSPATH . 'wp-admin/includes/misc.php';
            }

            if (!$this->is_apache() || !$this->htaccess_is_ok()) {
                return;
            }

            $htaccess = get_home_path() . ".htaccess";

            $write_rules = array();
            $write_rules[] = "# BEGIN WebP Images";
            $write_rules[] = "<IfModule mod_setenvif.c>
    # Vary: Accept for all the requests to jpeg, png, and gif.
    SetEnvIf Request_URI \"\.(jpg|jpeg|jpe|png|gif)$\" REQUEST_image
</IfModule>";
            $write_rules[] = "<IfModule mod_rewrite.c>
    RewriteEngine On
    RewriteBase /

    # Check if browser supports WebP images.
    RewriteCond %{HTTP_ACCEPT} image/webp

    # Check if WebP replacement image exists.
    RewriteCond %{REQUEST_FILENAME}.webp -f

    # Serve WebP image instead.
    RewriteRule (.+)\.(jpg|jpeg|jpe|png|gif)$ $1.$2.webp [T=image/webp,NC]
</IfModule>";
            $write_rules[] = "<IfModule mod_headers.c>
    Header append Vary Accept env=REQUEST_image
</IfModule>";
            $write_rules[] = "<IfModule mod_mime.c>
    AddType image/webp .webp
</IfModule>";
            $write_rules[] = "# END WebP Images";
            $write_rules[] = "\n";


            $oldContents = file_get_contents($htaccess);
            $fr = @fopen($htaccess, 'w');
            fwrite($fr, implode("\n", $write_rules));
            fwrite($fr, $oldContents);
            fclose($fr);
        }

        public function remove_htaccess_rules()
        {
            if (!function_exists('get_home_path')) {
                require_once ABSPATH . 'wp-admin/includes/file.php';
            }

            if (!$this->is_apache() || !$this->htaccess_is_ok()) {
                return;
            }

            $htaccess = get_home_path() . ".htaccess";
            $markerdata = explode("\n", implode('', file($htaccess)));
            $found = false;
            $newdata = '';
            foreach ($markerdata as $line) {
                if ($line == '# BEGIN WebP Images') {
                    $found = true;
                }
                if (!$found) {
                    $newdata .= "{$line}\n";
                }
                if ($line == '# END WebP Images') {
                    $found = false;
                }
            }
            // write back
            $f = @fopen($htaccess, 'w');
            if (fwrite($f, $newdata)) {
                return true;
            } else {
                return false;
            }
        }

        public function plugin_enable()
        {
            $this->stop_cron_schedule();
            $this->write_htaccess_rules();
        }

        public function plugin_reset()
        {
            $this->stop_cron_schedule();
            $this->remove_htaccess_rules();
        }

        public function start_cron_schedule($schedule)
        {
            wp_schedule_event(time(), 'webp_images_bulk', $schedule);
            return;
        }

        public function stop_cron_schedule()
        {
            if (wp_next_scheduled('init_bulk_webp_conversion')) {
                wp_clear_scheduled_hook('init_bulk_webp_conversion');
            }
            if (wp_next_scheduled('init_bulk_webp_remove')) {
                wp_clear_scheduled_hook('init_bulk_webp_remove');
            }
            return;
        }

        public function add_cron_schedules($schedules)
        {
            $time_between_bulk_actions = apply_filters('webp_images_bulk_time_between', WEBP_IMAGES_BULK_TIME);

            $schedules['webp_images_bulk'] = array(
                'interval' => $time_between_bulk_actions,
                'display'  => sprintf(__('%s seconds'), $time_between_bulk_actions),
            );
            return $schedules;
        }

        public function write_log($log)
        {
            if (true === WP_DEBUG) {
                if (is_array($log) || is_object($log)) {
                    error_log(print_r($log, true));
                } else {
                    error_log($log);
                }
            }
        }

        public function webp_images_countdown()
        {
            if (!in_array($_REQUEST['type'], array('conversion', 'remove')) || !wp_verify_nonce($_REQUEST['token'], "webp-images-countdown")) {
                die();
            }
            $return = 0;
            if ('conversion' == $_REQUEST['type']) {
                $result = (int) $this->count_not_webp_media_images();
                $return = $return + $result;
            }
            if ('remove' == $_REQUEST['type']) {
                $result = (int) $this->count_webp_media_images();
                $return = $return + $result;
            }
            if (!empty($_SERVER['HTTP_X_REQUESTED_WITH']) && strtolower($_SERVER['HTTP_X_REQUESTED_WITH']) == 'xmlhttprequest') {
                echo $return;
            }
            die();
        }
    }
    $webp_images = new webp_images();
    $webp_images->init_plugin();
}
